# Efficiency Evalulaution with NGL202 Powersupply
# The script opens a connection to an NGL202 power supply and measures the efficiency of a 12V to 5V DC-DC power supply ( Max Power = 5W)
# A log file will be generated and a graph at three different voltages is presented at the end of the script
# Preconditions:
# - R&S VISA or NI VISA installed
# - Python 3.8 or newer installed
# - Python PyVISA 1.10.1 package or newer installed
# - Python numpy 1.18.3 package or newer installed
# - Python matplotlib 3.2.1 package or newer installed
# - Author : Marcus Sonst / Rohde&Schwarz / 2020/04/29




import sys
import pyvisa
import time
import numpy as np
import matplotlib.pyplot as plt
import math

resource_manager = pyvisa.ResourceManager()

# Input (All voltage and current values are in mV or mA)
inputChannelNumber = 1
inputVoltageMax = 16000
inputVoltageDefault = 12000
inputVoltageMin = 8000
inputVoltageParameter = [inputVoltageDefault, inputVoltageMax, inputVoltageMin]
inputCurrentMax = 1000

# Output (All voltage and current values are in mV or mA)
outputChannelNumber = 2
outputCurrentMax = 1000
outputCurrentMin = 50
outputCurrentStep = 10
# Initial values to have an easy startup condition for the converter
outputVoltageStart = 4750
outputCurrentStart = 1

# Number of measurements at one voltage parameter
measurementCounterMax = ((outputCurrentMax - outputCurrentMin) / outputCurrentStep) + 1
measurementCounter = 0

# Definitions for plot
power1 = np.zeros(int(measurementCounterMax))
efficiency1 = np.zeros(int(measurementCounterMax))
power2 = np.zeros(int(measurementCounterMax))
efficiency2 = np.zeros(int(measurementCounterMax))
power3 = np.zeros(int(measurementCounterMax))
efficiency3 = np.zeros(int(measurementCounterMax))

# VISA Resource String to be changed to match your instrument!
ngl202 = resource_manager.open_resource("TCPIP0::10.202.1.16::INST0::INSTR")
ngl202.timeout = 10000  # timeout in ms
print("Available:" + "\n" + ngl202.query("*IDN?"))

input("Prepare for Test - Press Enter to Continue")
print("Opening a logfile ... ")
f = open("efficiency.csv","a")

print("Preparing Instrument ...")
ngl202.write("*RST;*CLS")  # Reset the instrument, clear the Error queue
ngl202.write("SYST:BEEP:STAT OFF")  # Silence Mode

# Input channel configuration
ngl202.write("INST:NSEL "+str(inputChannelNumber))
ngl202.write("SENS:CURR:RANG:AUTO 0")
ngl202.write("SENS:CURR:RANG 20")
ngl202.write("OUTP:MODE SOUR")
ngl202.write("SOUR:VOLT "+str(inputVoltageDefault/1000.0))
ngl202.write("SOUR:CURR "+str(inputCurrentMax/1000.0))

# Output channel configuration
ngl202.write("INST:NSEL "+str(outputChannelNumber))
ngl202.write("SENS:CURR:RANG:AUTO 0")
ngl202.write("SENS:CURR:RANG 10")
ngl202.write("OUTP:MODE SINK")
ngl202.write("SOUR:VOLT "+str(outputVoltageStart/1000.0))
ngl202.write("SOUR:CURR "+str(outputCurrentStart/1000.0))

print("Switch on output and input channel ...")
ngl202.write("INST:NSEL "+str(outputChannelNumber))
ngl202.write("OUTP 1")
time.sleep(0.2)
ngl202.write("INST:NSEL "+str(inputChannelNumber))
ngl202.write("OUTP 1")
ngl202.query("*OPC?")

time.sleep(1.0)

for voltage in range(len(inputVoltageParameter)):
    ngl202.write("INST:NSEL "+str(inputChannelNumber))
    ngl202.write("SOUR:VOLT "+str(inputVoltageParameter[voltage]/1000.0))
    time.sleep(2.0)  # Wait to obtain a stable condition of converter after next voltage step

    measurementCounter = 0

    for current in range(outputCurrentMin, outputCurrentMax + 1, outputCurrentStep):
        ngl202.write("INST:NSEL "+str(outputChannelNumber))
        ngl202.write("SOUR:CURR "+str(current/1000.0))
        ngl202.query("*OPC?")
        time.sleep(0.4)  # Wait to obtain a stable condition after next current step

        ngl202.write("INST:NSEL " + str(inputChannelNumber))
        ngl202.write("MEAS:STAT:RES")
        ngl202.write("INST:NSEL " + str (outputChannelNumber))
        ngl202.write("MEAS:STAT:RES")

        time.sleep(1.1)  # Number of average values = 10
        result1 = ngl202.query("MEAS:STAT:COUN?")  # Number of Average values for channel 1
        result2 = ngl202.query("MEAS:STAT:COUN?")  # Number of Average values for channel 2
        while True:
             try:
                 ngl202.write("INST:NSEL "+str(inputChannelNumber))
                 inputVoltage = ngl202.query_ascii_values("MEAS:VOLT:AVG?", separator="\r")
                 inputCurrent = ngl202.query_ascii_values("MEAS:CURR:AVG?", separator="\r")
                 inputPower = ngl202.query_ascii_values("MEAS:POW:AVG?", separator="\r")
                 ngl202.write("INST:NSEL "+str(outputChannelNumber))
                 outputVoltage = ngl202.query_ascii_values("MEAS:VOLT:AVG?", separator="\r")
                 outputCurrent = ngl202.query_ascii_values("MEAS:CURR:AVG?", separator="\r")
                 outputPower = ngl202.query_ascii_values("MEAS:POW:AVG?", separator="\r")
                 if math.isnan(inputVoltage[0]) or math.isnan(inputCurrent[0]) or math.isnan(outputVoltage[0]) or \
                         math.isnan(outputCurrent[0]) or math.isnan(inputPower[0]) or math.isnan(outputPower[0]):
                     raise Exception("NaN value returned!")
             except:
                 print("Read Error. Retrying!")
                 continue
             else:
                 break

        efficiency = outputPower[0] / inputPower[0] * (-100)

        if inputVoltageParameter[voltage] == inputVoltageDefault:
            power1[measurementCounter] = inputPower[0]
            efficiency1[measurementCounter] = efficiency
        elif inputVoltageParameter[voltage] == inputVoltageMax:
            power2[measurementCounter] = inputPower[0]
            efficiency2[measurementCounter] = efficiency
        elif inputVoltageParameter[voltage] == inputVoltageMin:
            power3[measurementCounter] = inputPower[0]
            efficiency3[measurementCounter] = efficiency

        measurementCounter = measurementCounter + 1
        print(str(inputVoltageParameter[voltage]/1000.0)+","+str(current/1000.0)+","+str(inputVoltage[0])+","
              + str(inputCurrent[0])+","+str(outputVoltage[0])+","+str(-outputCurrent[0])+","+str(inputPower[0])+","
              + str(-outputPower[0])+","+str(efficiency))
        f.write(str(inputVoltageParameter[voltage]/1000.0)+","+str(current/1000.0)+","+str(inputVoltage[0])+","
                + str(inputCurrent[0])+","+str(outputVoltage[0])+","+str(-outputCurrent[0])+","+str(inputPower[0])+","
                + str(-outputPower[0])+","+str(efficiency)+"\n")
        f.flush()

# Clean-Up and switch off
print("Tests Completed - Exiting!")
f.close()
ngl202.write("OUTP:GEN 0")
ngl202.close()

# Show data in a xy plot
plt.ylabel("Efficiency [%]")
plt.xlabel("Input Power [W]")
plt.plot(power1, efficiency1, label="Vin = " + str(inputVoltageParameter[0]/1000.0) + " V")
plt.plot(power2, efficiency2, label="Vin = " + str(inputVoltageParameter[1]/1000.0) + " V")
plt.plot(power3, efficiency3, label="Vin = " + str(inputVoltageParameter[2]/1000.0) + " V")
plt.title("Efficiency Buck Converter")
plt.legend()
plt.show()
